package production

import (
	"shylinux.com/x/ice"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/operation/src/production/model"
)

type issue struct {
	Table
	task       task
	order      string `data:"2"`
	fields     string `data:"title,content,issue_type,level,issue_status,task_count,process_time,finish_time,plan_uid,user_uid"`
	create     string `name:"create title* content* issue_type:select level:select" role:"leader,worker"`
	modify     string `name:"modify title* content* issue_type*:select level*:select" role:"leader,worker"`
	remove     string `name:"remove" role:"leader,worker"`
	reject     string `name:"reject" role:"leader"`
	approve    string `name:"approve" role:"leader"`
	process    string `name:"process" role:"leader,worker"`
	finish     string `name:"finish" role:"leader,worker"`
	planBind   string `name:"planBind plan_uid*" role:"leader"`
	taskCreate string `name:"taskCreate title* content* begin_time:select@date end_time:select@date" role:"leader,worker"`
}

func (s issue) Create(m *ice.Message, arg ...string) {
	s.ValueCreate(m, kit.ArgDef(arg, kit.Simple(model.ISSUE_TYPE, IssueFeature, model.LEVEL, Level3)...)...)
	s.planCount(m, "1")
	s.DashboardUpdate(m)
	s.GetCommandUID(m)
	s.SendMessage(m, "", "")
}
func (s issue) Remove(m *ice.Message, arg ...string) {
	s.ValueRemove(m, arg...)
	s.planCount(m, "-1")
	s.DashboardUpdate(m)
}
func (s issue) List(m *ice.Message, arg ...string) {
	isLeader, user_uid := s.IsLeader(m), m.Option(model.USER_UID)
	s.ValueList(m, arg).Table(func(value ice.Maps) {
		button := []ice.Any{}
		switch IssueStatus(kit.Int(value[model.ISSUE_STATUS])) {
		case IssueCreate:
			if isLeader {
				button = append(button, s.Reject, s.Approve)
			}
		case IssueApproved:
			if isLeader && value[model.PLAN_UID] == "" {
				button = append(button, s.PlanBind)
			}
			if isLeader || user_uid == value[model.USER_UID] {
				button = append(button, s.Process)
			}
		case IssueProcess:
			button = append(button, s.TaskCreate)
			if isLeader || user_uid == value[model.USER_UID] {
				button = append(button, s.Finish)
			}
		case IssueFinish:
			kit.If(len(arg) == 2, func() { s.DoneMessage(m) })
		}
		m.PushButton(button...)
	}).Display("")
	s.SelectJoinPlan(m)
}
func (s issue) Reject(m *ice.Message, arg ...string) {
	s.changeStatus(m, IssueCreate, IssueRejected)
}
func (s issue) Approve(m *ice.Message, arg ...string) {
	s.changeStatus(m, IssueCreate, IssueApproved)
}
func (s issue) Process(m *ice.Message, arg ...string) {
	s.changeStatus(m, IssueApproved, IssueProcess)
}
func (s issue) Finish(m *ice.Message, arg ...string) {
	count := m.Cmd(s.task, s.Select, "issue_uid = ? AND status != ?", m.Option(model.UID), TaskFinish).Length()
	if m.WarnNotValid(count > 0, kit.Format("还有 %v 个未完成的任务", count)) {
		return
	}
	s.changeStatus(m, IssueProcess, IssueFinish)
	s.DashboardUpdate(m)
}
func (s issue) PlanBind(m *ice.Message, arg ...string) {
	s.ValueModify(m, arg...)
	s.planCount(m, "1")
}
func (s issue) TaskCreate(m *ice.Message, arg ...string) {
	m.Cmd(s.task, s.Create, arg, model.ISSUE_UID, m.Option(model.UID), m.OptionSimple(model.PLAN_UID))
}

func init() { ice.TeamCtxCmd(issue{}) }

func (s issue) changeStatus(m *ice.Message, from, to IssueStatus) {
	s.ChangeStatus(m, int(from), int(to), m.ActionKey()+"_time", m.Time())
}
func (s issue) planCount(m *ice.Message, value string) {
	s.Table.planCount(m, model.ISSUE_COUNT, value)
}

type IssueType int

const (
	IssueFeature IssueType = iota
	IssueBugfix
)

var IssueTypeList = map[IssueType]string{
	IssueFeature: "feature",
	IssueBugfix:  "bugfix",
}

func (s IssueType) String() string { return IssueTypeList[s] }

type IssueStatus int

const (
	IssueCreate IssueStatus = iota
	IssueRejected
	IssueApproved
	IssueProcess
	IssueFinish
)

var IssueStatusList = map[IssueStatus]string{
	IssueCreate:   "create",
	IssueRejected: "rejected",
	IssueApproved: "approved",
	IssueProcess:  "process",
	IssueFinish:   "finish",
}

func (s IssueStatus) String() string { return IssueStatusList[s] }
